{% extends base_change_form_template %}
{% comment %}
  In case no page template was created (which the admin auto-selects by path),
  this template will work as catch-all, and choose the proper base template
  that the current admin class needs.
{% endcomment %}
