# import apipkg
#
# realmodule = "fluent_pages.integration._fluent_contents"
# apipkg.initpkg(__name__, {
#     'fluent_contents': {
#         'FluentContentsPage': realmodule + ".models.FluentContentsPage",
#         'FluentContentsPageAdmin': realmodule + ".admin.FluentContentsPageAdmin",
#         'FluentContentsPagePlugin': realmodule + ".admin.FluentContentsPagePlugin",
#     }
# })
