{% extends FLUENT_PAGES_BASE_TEMPLATE %}

{% block meta-keywords %}{{ page.keywords }}{% endblock %}
{% block meta-description %}{{ page.description }}{% endblock %}

{% block title %}{{ page.title }}{% endblock %}

{% block content %}
  {{ page.content }}
{% endblock %}
